from pathlib import Path

from p1st.exec_print_capture import execute_print_capture
from p1st.unix_sock_input import wait_for_message


def repeat_until_successful(command: list[str],
                            socket_file: Path) -> str:
    """
    Executes the given `command`.

    If an error occurs, it creates a UNIX socket at
    `socket_file` and waits for user input.
    """
    while True:
        returncode: int
        out: str
        returncode, out, _err = execute_print_capture(command)
        if returncode == 0:
            return out

        print(f'\n'
              f'Error while executing:\n'
              f'\t{command}\n'
              f'\tFor details, see above output.')

        print(f'Info:\n'
              f'\tPlease fix the above error first. Then continue here:\n'
              f'\tsudo pacman -S --needed openbsd-netcat\n'
              f'\techo "OK" | nc -U "{socket_file.absolute()}"')
        wait_for_message(socket_file, [b'OK\n'])
        print()
